-- voxel shader
-- (c) David Balaun 2018
-- No part of this software may used, reproduced, or distributed without permission.


local voxel = love.graphics.newShader([[
		extern vec2 camRot;
		extern vec2 objRot;
		extern vec2 pos;
		extern int lightCount;
		extern vec3[4] lightDir;
		extern vec3[4] lightCol;
		extern vec3 ambColor;
		extern VolumeImage img;
		mat3 rx(in number r)
		{
			mat3 mt;
			number c = cos(r); 
			number s = sin(r);
			mt[0] = vec3(1,0,0);
			mt[1] = vec3(0,c,-s);
			mt[2] = vec3(0,s,c);
			return mt;
		}
		mat3 rz(in number r)
		{
			mat3 mt;
			number c = cos(r); 
			number s = sin(r);
			mt[0] = vec3(c,s,0);
			mt[1] = vec3(-s,c,0);
			mt[2] = vec3(0,0,1);
			return mt;
		}
		int check(vec3 ray)
		{
			number u = 1.1;
			number l = -0.1;
			if (ray.x > u || ray.x < l)
				return 0;
			else if (ray.y > u || ray.y < l)
				return 0;
			else if (ray.z > u || ray.z < l)
				return 0;
			else
				return 1;
		}
		vec4 effect( vec4 color, Image texture, vec2 tc, vec2 sc )
        {
			number alpha = 0.85;
			number size = 2.0;
			vec2 pixPos = tc-pos;		
			vec3 ro = vec3(0.0,0.0,size);
			vec3 rd = normalize(vec3(pixPos,-1.0));
			mat3 orx = rx(objRot.y); 
			mat3 orz = rz(objRot.x);
			mat3 crx = rx(camRot.y); 
			mat3 crz = rz(camRot.x);
			ro*=orx;
			ro*=orz;
			rd*=orx;
			rd*=orz;			
			ro*=crx;
			ro*=crz;
			rd*=crx;
			rd*=crz;
			int i = 0;
			vec4 sample = vec4(0.0);
			vec3 acol = ambColor;
			ro+=0.5;
			number stepLarge = 0.022;
			number stepSmall = 0.008;
			while (i < 220)
			{
				if( check(ro) == 1)
				{
					vec3 rn = ro;
					ro += rd*stepSmall; // make step smaller since we are inside the image
					sample = Texel(img,ro);
					if(sample.a > alpha)
					{	
						vec4 sampleBack = Texel(img,rn);
						number alphaMul = 1-(sample.a-alpha)/(sample.a-sampleBack.a);
						ro=rn+rd*alphaMul*stepSmall;
						sample = Texel(img,ro);
						for(int l = 0; l < lightCount; l++)
						{
							vec3 lp = vec3(ro);
							vec3 ldir = normalize(lightDir[l]);
							ldir*=orx;
							ldir*=orz;
							lp+=ldir*0.01;
							vec4 newSample = Texel(img,lp);
							number lightAngle = clamp((sample.a-newSample.a)*5.0,0.0,1.0);
							if (lightAngle > 0.0)
							{
								number st = 0;
								while(st < 100)
								{
									st+=1;
									lp+=ldir*0.02;
									newSample = Texel(img,lp);
									if(newSample.a > alpha)
											break;
								}
								number shadow = st/100;
								if (shadow > 0.0)
									acol += lightCol[l]*min(lightAngle*shadow,1);
							}
						}
						sample.rgb *= acol;
						break;
					}
				}
				else
					ro+=rd*stepLarge; // big steps if we are outside image area
				i++;
			}
			return sample;
			
		}
    ]])
	
return voxel