-- voxel shader
-- (c) David Balaun 2018
-- No part of this software may used, reproduced, or distributed without permission.

local voxelShader = require "shaders.voxel"
local voxelStorageTable = {}
local voxelCanvas =  love.graphics.newCanvas(1000,1000)
local workingCanvas =  love.graphics.newCanvas(1000,1000)
local voxel = {}
local lightDir = {}
local lightCol = {}
function voxel.clearLights()
	lightDir = {}
	lightCol = {}
end
function voxel.setAmbLight(color)
	voxelShader:send("ambColor",color)	
end
function voxel.addLight(dir,color)
	local i = #lightDir+1
	lightDir[i] = dir
	lightCol[i] = color
end

function voxel.sendLights()
	for s = 1, 4 do
		if not lightDir[s] then lightDir[s] = {0,0,0} end
		if not lightCol[s] then lightCol[s] = {0,0,0} end
	end
	voxelShader:send("lightCount",#lightDir)	
	voxelShader:send("lightDir",lightDir[1],lightDir[2],lightDir[3],lightDir[4])	
	voxelShader:send("lightCol",lightCol[1],lightCol[2],lightCol[3],lightCol[4])	
end

function voxel.print()
	love.graphics.print(#lightCol,10,30)
end

function voxel.rotate(rx,rz)
	voxelShader:send("objRot",{rx,rz})	
end

function voxel.draw(fileName,x,y,rx,rz,scale)
	x = x or 0
	y = y or 0
	rx = rx or 0
	rz = rz or 0
	cx = cx or 0
	cz = cz or 0
	scale = scale or 1
	local vox = voxel.get(fileName)		
	voxelShader:send("camRot",{rx,rz})			
	voxelShader:send("pos",{.5,.5})								
	voxelShader:send("img",vox)	
	love.graphics.setShader(voxelShader)
	love.graphics.draw(voxelCanvas,x,y,0,scale,scale,500,500)
	love.graphics.setShader()
end

function voxel.load(fileName,wc,hc)
	wc = wc or 1
	hc = hc or 1
	if not voxelStorageTable[fileName] then
		local images = voxel.sectionImage(fileName,wc,hc)
		voxelStorageTable[fileName] = love.graphics.newVolumeImage(images,{mipmaps = true})
	end
end
function voxel.get(fileName)
	return voxelStorageTable[fileName]
end


function voxel.sectionImage(fileName,wc,hc)
	local data = love.image.newImageData(fileName)
	local tbl = {}
	local width, height = data:getDimensions( )
	hc = hc or 1
	wc = wc or width/height
	local w = width/wc
	local h = height/hc
	local p = 1 -- padding in pixels
	local pw = w + 2*p
	local ph = h + 2*p
	if p > 0 then
		for i = 1,p do
			tbl[#tbl+1] = love.image.newImageData(pw, ph)
		end
	end
	for x = 0, wc-1 do
		for y = 0,hc-1 do
			local i = #tbl+1
			tbl[i] = love.image.newImageData(pw, ph)
			for px = 0,w-1 do
				for py = 0,h-1 do
					local  r, g, b, a  = data:getPixel(px+x*w, py+y*h)
					tbl[i]:setPixel( px+p, py+p, r, g, b, a )
				end
			end
			 tbl[i]:encode( "png", i..".png" )
		end
	end
	if p > 0 then
		for i = 1,p do
			tbl[#tbl+1] = love.image.newImageData(pw, ph)
		end
	end
	return tbl
end




return voxel